import QtQuick 2.5
import com.zuragon.ViCANdo 1.0
import QtQuick.Extras 1.4
import QtQuick.Controls 1.4
import QtQuick.Layouts 1.3


Rectangle {
        width: 800
        height: 600
        color: "white"
        id: leddar_tech_vu8
        visible: true

        property int number_of_detections;
        property int current_light_source_power;
        property int acquisition_timestamp;

        property int distance;
        property double amplitude;
        property int flag;
        property int segment_number;

        property string two_spaces: " - ";

        Text{
            id: title
            text: " ViCANdo QML example for LeddarTech Vu8 "
            font.family: "Helvetica"
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
            font.pointSize: 18
        }

        GroupBox {
            id: group_box_config
            title: "QML Demo Configuration to match LeddarTech Vu8"
            anchors.top: title.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5

            RowLayout {
                id: first_row
                ExclusiveGroup { id: single_msg_mode_group }
                RadioButton {
                    id: single_button
                    checked: true
                    text: "single-message ID mode"
                    exclusiveGroup: single_msg_mode_group
                }
                RadioButton {
                    id: multiple_button
                    text: "multiple-message ID mode"
                    exclusiveGroup: single_msg_mode_group
                }
            }

            RowLayout {
                id: second_row
                anchors.top: first_row.bottom
                anchors.topMargin: 5
                Text{
                    id: tx_base_id_label
                    text: " Tx base ID(dec):"
                    font.family: "Helvetica"
                    anchors.left: parent.left
                    anchors.leftMargin: 5
                }
                TextField{
                    id: tx_base_id
                    text: "1872"
                    font.family: "Helvetica"
                    anchors.left: tx_base_id_label.right
                    anchors.leftMargin: 5
                }
            }

            RowLayout {
                id: third_row
                anchors.top: second_row.bottom
                anchors.topMargin: 5
                Text{
                    id: rx_base_id_label
                    text: " Rx base ID(dec):"
                    font.family: "Helvetica"
                    anchors.left: parent.left
                    anchors.leftMargin: 5

                }
                TextField{
                    id: rx_base_id
                    text: "1856"
                    font.family: "Helvetica"
                    anchors.left: rx_base_id_label.right
                    anchors.leftMargin: 5
                }
            }

            RowLayout{
                id: fourth_row
                anchors.top: third_row.bottom
                anchors.topMargin: 5

                Button {
                    id: set_button
                    text: "Set"
                    anchors.left: parent.left
                    anchors.leftMargin: 5

                    onClicked: {
                        config_result_string.text = "Config done. Tx base ID is: 0x" + parseInt(tx_base_id.text, 10).toString(16) + ", Rx base ID is: 0x" + parseInt(rx_base_id.text, 10).toString(16)
                        config_result_string.color = "green"

                    }
                }
                Text{
                    id: config_result_string
                    text: "<-- Click this button to save the configuration BEFORE playing the session."
                    color: "red"
                    font.family: "Helvetica"
                    anchors.left: set_button.right
                    anchors.leftMargin: 5

                }

            }

        }

        Rectangle {
            id: line_separator
            height: 2
            color: "#000000"
            anchors.top: group_box_config.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.left: parent.left
            anchors.leftMargin: 0
        }

        Text {
            id: number_of_detections_label
            text: "Number of detections "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: line_separator.bottom
            anchors.topMargin: 10
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: number_of_detections_value
            text: number_of_detections
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: number_of_detections_label.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 65
        }
        Text {
            id: current_light_source_power_label
            text: "Current light source power"
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: line_separator.bottom
            anchors.topMargin: 10
            anchors.horizontalCenter: parent.horizontalCenter
        }
        Text {
            id: current_light_source_power_value
            text: current_light_source_power + " %"
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: current_light_source_power_label.bottom
            anchors.topMargin: 5
            anchors.horizontalCenter: parent.horizontalCenter
        }

        Text {
            id: acquisition_timestamp_label
            text: "Acquisition timestamp "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: line_separator.bottom
            anchors.topMargin: 10
            anchors.right: parent.right
            anchors.rightMargin: 5
        }
        Text {
            id: acquisition_timestamp_value
            text:acquisition_timestamp + " ms"
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: acquisition_timestamp_label.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 65
        }


        Rectangle {
            id: line_separator_2
            height: 2
            color: "#000000"
            anchors.top: current_light_source_power_value.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            anchors.rightMargin: 0
            anchors.left: parent.left
            anchors.leftMargin: 0
        }


        Text {
            id: detection_header_text
            text: "Segment  Distance  Amplitude  Flag"
            font.family: "Helvetica"
            font.pointSize: 10
            anchors.top: line_separator_2.bottom
            anchors.topMargin: 10
            anchors.left: parent.left
            anchors.leftMargin: 5
        }

        Text {
            id: segment_1
            text:  "1   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: detection_header_text.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_2
            text:  "2   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_1.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }

        Text {
            id: segment_3
            text:  "3   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_2.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_4
            text:  "4   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_3.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_5
            text:  "5   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_4.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_6
            text:  "6   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_5.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_7
            text:  "7   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_6.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }
        Text {
            id: segment_8
            text:  "8   "
            font.family: "Helvetica"
            font.pointSize: 12
            anchors.top: segment_7.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.leftMargin: 5
        }


        SourceEventListener {
                        id: listen_to_can_msg
                        source: project.findObject("can0") // can0 is the first CAN source
                        dbc: project.findObject("leddartech_vu8_dbc") // note this database is only for single msg-ID mode
                        onFrameDataReceived: {

                            //project.log("Rx msg id: " + _data.id + " dlc: " + _data.dlc + " data: " + _data.data)

                            if( _data.id == parseInt(rx_base_id.text, 10)){
                              // leddarTech Vu8 rx message, request from the host
                            }
                            if( _data.id == parseInt(tx_base_id.text, 10)){
                              // leddarTech Vu8 tx message, answer to the host, add your own code to decode the contents of the message
                            }

                            if( _data.id == parseInt(tx_base_id.text,10) + 1){
                                // Message ID(Tx base ID + 1) received, contains info like number of detections
                                number_of_detections = _data.data[0]
                                current_light_source_power = _data.data[1]
                                acquisition_timestamp = (_data.data[7]<<24) | (_data.data[6]<<16) | (_data.data[5]<<8) | _data.data[4]
                                /*
                                project.log("number_of_detections is: " + number_of_detections)
                                project.log("light source power is: " + current_light_source_power + " %")
                                project.log(" timestamp: " + acquisition_timestamp + " ms")
                                */
                            }

                            // If the sensor is configured to send message in single message ID mode
                            if( single_button.checked){
                                // Decode the detection info from the single message whose ID is Tx base ID + 2
                                if( _data.id == parseInt(tx_base_id.text,10) + 2){

                                    distance = ((_data.data[1]<<8) | _data.data[0]).toFixed(2)
                                    amplitude = (((_data.data[3]<<8) | _data.data[2])/64).toFixed(2)
                                    flag = _data.data[4];
                                    segment_number = ( _data.data[7]<< 8) | _data.data[6]
                                    project.log("Single-msg: id: " + _data.id + ", seg_num: " + segment_number + ", distance: " + distance)

                                    if( segment_number == 1){
                                        segment_1.text = "1" + two_spaces +  distance +  two_spaces + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 2){
                                        segment_2.text = "2" + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 3){
                                        segment_3.text = "3" + two_spaces + distance +   two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 4){
                                        segment_4.text = "4" + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 5){
                                        segment_5.text = "5"  + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 6){
                                        segment_6.text = "6"  + two_spaces + distance +  two_spaces + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 7){
                                        segment_7.text = "7"  + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 8){
                                        segment_8.text = "8" + two_spaces + distance +   two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                }

                            }
                            // If the sensor is configured in multiple-message ID mode
                            else{
                                var msg_id_range_end   = parseInt(tx_base_id.text,10) + 2 + number_of_detections
                                var msg_id_range_start = parseInt(tx_base_id.text,10) + 2
                                if( _data.id >= msg_id_range_start && _data.id <= msg_id_range_end){

                                    distance = ((_data.data[1]<<8) | _data.data[0]).toFixed(2)
                                    amplitude = (((_data.data[3]<<8) | _data.data[2])/64).toFixed(2)
                                    flag = _data.data[4] // byte 4 and 5 are flag.
                                    segment_number = ( _data.data[7]<< 8) | _data.data[6]

                                    //project.log("Multi-msg: id: " + _data.id + ", seg_num: " + segment_number + ", distance: " + distance)
                                    if( segment_number == 1){
                                        segment_1.text = "1" + two_spaces +  distance +  two_spaces + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 2){
                                        segment_2.text = "2" + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 3){
                                        segment_3.text = "3" + two_spaces + distance +   two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 4){
                                        segment_4.text = "4" + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 5){
                                        segment_5.text = "5"  + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 6){
                                        segment_6.text = "6"  + two_spaces + distance +  two_spaces + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 7){
                                        segment_7.text = "7"  + two_spaces + distance +  two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                    else if( segment_number == 8){
                                        segment_8.text = "8" + two_spaces + distance +   two_spaces  + amplitude + two_spaces + getFlagInfoString(flag)
                                    }
                                }

                            }
                        }
        }

        function getFlagInfoString(flag){
            var flag_detection_valid = flag & 1
            var flag_detection_is_object_demerging_result = flag & 2
            var flag_detection_is_saturated = flag & 8
            var res

            if(flag_detection_valid == 1){
                res = "detection is valid"
            }else{
                res = "detection is invalid"
            }
            if(flag_detection_is_object_demerging_result == 1){
                res = res + two_spaces +  "is the result of object demerging"
            }
            if(flag_detection_is_saturated == 1){
                res = res + two_spaces + "is saturated"
            }
            return res
        }

/*

// Table 51, get function request description
function getFunctionRequestDescription( function_request){
    switch(function_request){
    case 1  : return "Stop sending detections continuously";
    case 2  : return "Send detection once";
    case 3  : return "Start sending detections continuously";
    case 4  : return "Get input data (read only)";
    case 5  : return "Get holding data";
    case 6  : return "Set holding data";
    case 7  : return "Set base address";
    case 8  : return "Read module data";
    case 9  : return "Write module data";
    case 10  : return "Send module opcode command";
    default:
        return "Invalid Function Request";
    }
}

// Table 52,  CAN bus request message (Get input data)
function getInputDataDescription(input_data_type){
    switch(input_data_type){
    case 0 : return "Number of segments";
    case 1 : return "Device identificaiton and option";
    case 2 :
    case 3 : return "Firmware version";
    case 4 :
    case 5 : return "Bootloader version";
    case 6 : return "FPGA version";
    case 7 :
    case 8 :
    case 9 :
    case 10 :
    case 11 :
    case 12 : return "Serial number";
    case 13 :
    case 14 :
    case 15 :
    case 16 :
    case 17 :
    case 18 : return "Device name";
    case 19 :
    case 20 :
    case 21 :
    case 22 :
    case 23 :
    case 24 : return "Hardware part number";
    case 25 :
    case 26 :
    case 27 :
    case 28 :
    case 29 :
    case 30 : return "Software part number";
    default:
        return "Invalid Input Data Type";
    }
}

// Table 53, CAN bus request message (Get holding data)
function getHoldingDataDescription( holding_data_type){
    switch( holding_data_type){
    case 0: return "Acquisition configuration";
    case 1: return "Smoothing and detection threshold";
    case 2: return "Light source power management";
    case 3: return "Distance resolution and acquisition options";
    case 4: return "CAN port configuration 1";
    case 5: return "CAN port configuration 2";
    case 6: return "CAN port configuration 3";
    case 7: return "Reserved";
    case 8: return "Segment enabled";
    default: return "Invalid Holding Data Type";
    }
}


// Table 59, CAN bus answer message
function getAnswerDataDescription(answer_data){
    switch(answer_data){
    case 1: return "Answer to stop continuously sending detection requests";
    case 2: return "Answesr to send once a dtection request";
    case 3: return "Answer to continuously sending a detection request";
    case 4: return "Answer to the Get input data request";
    case 5: return "Answer to the Get holding data request";
    case 6: return "Answer to the Set holding data request";
    case 7: return "Answer to the Set base address request";
    case 8: return "Answer to the Read module data request";
    case 9: return "Answer to the Write module data request";
    case 10: return "Answer to the Send module opcode request";
    default: return "Invalid Answer Data";
    }
}


*/


}
